// robinsonizer engine by Bob Polis
// copyright (c) 1994-2019

#ifndef __engine__
#define __engine__

#include <string>
#include <vector>
#include <ostream>
#include <random>
#include "robinsonizer_mode.hpp"

class engine {
public:
	engine(std::string start,
            int maxiter,
            int maxseed,
            std::string numerals_file,
            std::string characters_file,
            unsigned int vl,
            robinsonizer_mode mode,
            bool easy_parsing,
            int engine_id);
	
	engine() = delete;

	void run();
	void write_result(std::ostream& os) const;
	void frequencies(std::ostream& os, const int fm[]) const;
	void numeral_frequencies(std::ostream& os) const;
	unsigned long long total_iterations() const {return _total_iterations; }
	bool found() const { return _found; }
	
protected:
	std::string							_start;
	int									_maxiter;
	unsigned long long					_total_iterations {0};
	std::vector<std::string>			_numerals;
    std::vector<std::vector<int>>       _frequencies; // for memory mgmt only
    std::vector<int*>           		_freq;
    std::vector<int>					_start_freq;
    std::vector<int>					_old;
    std::vector<int>					_new;
    std::vector<int>					_used;
	unsigned int						_verbosity_level;
	bool								_found {false};
	robinsonizer_mode					_mode;
	bool								_easy_parsing;
	int									_engine_id;
	std::default_random_engine			_random_engine {std::random_device {}()};
	std::uniform_int_distribution<int>	_dist;
};

std::ostream& operator<<(std::ostream& os, const engine& engine);

#endif // __engine__
